"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildCommand = exports.buildOption = exports.registerLongName = exports.isOptionAlreadyRegistered = exports.getShortFlagForLongName = exports.getCommandNameForShortFlag = exports.createCommand = void 0;
const commander_1 = require("commander");
const types_1 = require("../types");
const root_1 = require("../commands/handler/root");
const logger_1 = require("../logger");
const registeredOptions = {};
const getLogger = () => (0, logger_1.get)("utils.commands");
const checkEmpty = (name, value) => {
    if (value.trim() === "") {
        throw new Error(`missing value for parameter ${name}`);
    }
};
const checkCommand = (command) => {
    const { command: name, description } = command;
    try {
        checkEmpty("command", name);
        checkEmpty("description", description);
    }
    catch (err) {
        const { trace } = getLogger();
        trace(`failed to check command ${JSON.stringify(command, null, 2)} because of`, err);
        throw err;
    }
};
const createCommand = (name) => {
    const { output } = getLogger();
    const command = new commander_1.Command(name);
    command.enablePositionalOptions();
    command.exitOverride();
    command.configureOutput({ writeErr: output, writeOut: output });
    command.allowUnknownOption();
    return command;
};
exports.createCommand = createCommand;
function getCommandNameForShortFlag(commandName, option) {
    return option.commandName ?? commandName;
}
exports.getCommandNameForShortFlag = getCommandNameForShortFlag;
const getShortFlagForLongName = (commandName, option) => {
    const cName = getCommandNameForShortFlag(commandName, option);
    for (const s of Object.keys(registeredOptions[cName] ?? {})) {
        if (registeredOptions[cName][s] === option.longName) {
            return s;
        }
    }
    throw new Error(`no short flag registered for command ${cName}, long name ${option.longName}`);
};
exports.getShortFlagForLongName = getShortFlagForLongName;
const isOptionAlreadyRegistered = ({ longName }, command) => {
    const opts = command.options?.map((o) => o.long?.replace("--", ""));
    return !!opts?.includes(longName);
};
exports.isOptionAlreadyRegistered = isOptionAlreadyRegistered;
const registerLongName = (commandName, option) => {
    const cName = getCommandNameForShortFlag(commandName, option);
    try {
        return (0, exports.getShortFlagForLongName)(cName, option);
    }
    catch (err) {
        for (const c of [
            option.longName[0],
            option.longName[0].toUpperCase() === option.longName[0]
                ? option.longName[0].toLowerCase()
                : option.longName[0].toUpperCase(),
            ...types_1.CHARACTERS,
        ]) {
            if (!registeredOptions[cName]?.[c]) {
                registeredOptions[cName] = registeredOptions[cName] ?? {};
                registeredOptions[cName][c] = option.longName;
                return c;
            }
        }
        throw new Error(`no short flag found for name ${option.longName}, command ${cName}`);
    }
};
exports.registerLongName = registerLongName;
const buildOption = async (commandName, option) => {
    const mandatory = !!option.required;
    const shortFlag = (0, exports.registerLongName)(commandName, option);
    let newDescr = option.description;
    if (!mandatory) {
        newDescr = newDescr ? `${newDescr} (optional)` : "(optional)";
    }
    const argStr = option.args
        ? option.args.reduce((p, c) => (c.optional ? `${p} [${c.name}]` : `${p} <${c.name}>`), "")
        : "";
    const cOption = new commander_1.Option(`-${shortFlag}, --${option.longName}${argStr}`, newDescr)
        .makeOptionMandatory(mandatory)
        .hideHelp(!!option.hidden);
    if (option.default) {
        const defaultStr = typeof option.default === "function"
            ? await option.default()
            : option.default;
        cOption.default(defaultStr);
    }
    let choicesArr = [];
    if (option.choices) {
        choicesArr =
            typeof option.choices === "function"
                ? await option.choices()
                : option.choices;
    }
    if (choicesArr.length > 0) {
        cOption.choices(choicesArr);
    }
    return cOption;
};
exports.buildOption = buildOption;
const buildCommand = async (topCommand, command) => {
    []
        .concat([command], command.type === "topCommand" ? command.subCommands : [])
        .forEach((c) => checkCommand(c));
    const { command: name, description, options } = command;
    const comm = (0, exports.createCommand)(name);
    comm.description(description);
    comm.enablePositionalOptions();
    for (const option of options ?? []) {
        // eslint-disable-next-line no-await-in-loop
        comm.addOption(await (0, exports.buildOption)(name, option));
    }
    if (command.type === "topCommand") {
        const { subCommands } = command;
        for (const subCommand of subCommands) {
            // eslint-disable-next-line no-await-in-loop
            await (0, exports.buildCommand)(comm, subCommand);
        }
    }
    else {
        // command.type === "command"
        const { args, handler } = command;
        if (args) {
            args.forEach((a) => comm.argument(`<${a.argument}>`, a.description));
        }
        const rootHandler = await (0, root_1.create)(handler, `Failed to ${command.description}`)(comm);
        comm.action(rootHandler);
    }
    topCommand.addCommand(comm);
    return comm;
};
exports.buildCommand = buildCommand;
